package geoffhom.com.pKaValues {
	import flash.display.Bitmap;
	import flash.display.Loader;
	import flash.events.Event;
	import flash.events.EventDispatcher;
	import flash.events.IOErrorEvent;
	import flash.net.URLRequest;
	import flash.system.Security;

	/* Molecular compound.
	*/
	public class Compound extends EventDispatcher {
		
		/* Some compounds have images. The images are in one directory locally
		and another directory remotely.
		*/
		public static const LOCAL_PATH:String = "../../images/";
		public static const REMOTE_PATH:String = "images/";
		
		/* Compound data is in XML format. The current format is:
		<compounds>
		  <COMPOUND_TAG>
		    <NAME_TAG>name</NAME_TAG>
			<PKA_VALUE_TAG>pKa value</PKA_VALUE_TAG>
			<IMAGE_VALUE_TAG>image filename</IMAGE_VALUE_TAG>
		  </COMPOUND_TAG>
		  ...
		  <COMPOUND_TAG></COMPOUND_TAG>
		</compounds>
		*/
		public static const COMPOUND_TAG:String = "compound";
		public static const NAME_TAG:String = "name";
		public static const PKA_VALUE_TAG:String = "pKaValue";
		public static const IMAGE_VALUE_TAG:String = "image";

		public static const IMAGE_IO_ERROR:String = "Compound image IO error.";
		public static const IMAGE_LOADED:String = "Compound image loaded.";
		
		private var _name:String;
		private var _pKaValue:Number;
		private var _imageFilename:String;

		// Whether this compound has an image file. May or may not be loaded yet.
		private var _hasImage:Boolean;
		
		private var _loader:Loader;
		private var _url:String;
		private var _image:Bitmap;
		
		public function Compound(xml:XML) {
			
			_name = xml.elements(NAME_TAG);
			_pKaValue = xml.elements(PKA_VALUE_TAG);
			_imageFilename = xml.elements(IMAGE_VALUE_TAG);
			
			if (_imageFilename != "")
				_hasImage = true;
			else
				_hasImage = false;
				
			_image = null;
		}
		
		// This should match the XML tag, for consistency.
		// I am not sure how to enforce/check for this in the code.
		public function get name():String {
			return _name;
		}
		
		// This should match the XML tag, for consistency.
		public function get pKaValue():Number {
			return _pKaValue;
		}
		
		public function get hasImage():Boolean {
			return _hasImage;
		}
		
		//[Bindable("imageChangedTest")]
		public function get image():Bitmap {
			return _image;
		}

		// Try to load this compound's image file.
		public function loadImage():void {
			
			var path:String;
			if (Security.sandboxType == Security.LOCAL_TRUSTED)
				path = LOCAL_PATH;
			else
				path = REMOTE_PATH;
				
			_url = path + _imageFilename;
			_loader = new Loader();
			_loader.contentLoaderInfo.addEventListener(IOErrorEvent.IO_ERROR, handleIOError, false, 0, true);
			_loader.contentLoaderInfo.addEventListener(Event.COMPLETE, handleLoadingComplete, false, 0, true);
			var urlRequest:URLRequest = new URLRequest(_url);
			_loader.load(urlRequest);
		}
		
		// If the image was found, then set it and notify others.
		private function handleIOError(event:IOErrorEvent):void {
			
			event.currentTarget.removeEventListener(event.type, arguments.callee);
			event.currentTarget.removeEventListener(Event.COMPLETE, handleLoadingComplete);
			main.logThis("Could not find image: " + _url + ".");
			_hasImage = false;
			notifyOfImageIOError();
		}
		
		// If the image was found, then set it and notify others.
		private function handleLoadingComplete(event:Event):void {
			
			event.currentTarget.removeEventListener(event.type, arguments.callee);
			event.currentTarget.removeEventListener(IOErrorEvent.IO_ERROR, handleIOError);
			main.logThis("Image loaded: " + _url + ".");
			_image = Bitmap(_loader.content);
			//dispatchEvent(new Event("imageChangedTest") );
			notifyOfImageLoaded();
		}
		
		private function notifyOfImageIOError():void {
		
			dispatchEvent(new Event(Compound.IMAGE_IO_ERROR) );
		}
		
		private function notifyOfImageLoaded():void {
			
			dispatchEvent(new Event(Compound.IMAGE_LOADED) );
		}
	}
}