package geoffhom.com.pKaValues {
	import flash.events.EventDispatcher;
	import flash.events.StatusEvent;
	
	import mx.collections.ArrayCollection;
	import mx.collections.XMLListCollection;

	/* The Model contains all of the data. It is Bindable, so any
	changes should be immediately conveyable to other references.
	*/
	[Bindable]
	public class Model extends EventDispatcher {
		
		/* Reference data is in XML format. The current format is:
		<references>
		  <REFERENCE_TAG>
		    <TEXT_TAG>text</TEXT_TAG>
		  </REFERENCE_TAG>
		  ...
		  <REFERENCE_TAG></REFERENCE_TAG>
		</references>
		*/
		public static const REFERENCE_TAG:String = "reference";
		public static const TEXT_TAG:String = "text";
		
		// For loading data.
		private var _pKaLoader:PKaLoader;
		
		// For e-mailing messages.
		private var _emailer:Emailer;

		private var _referencesXMLListCollection:XMLListCollection;
		
		private var _compoundsArrayCollection:ArrayCollection;
		
		// For storing the compounds in a random order.
		private var _compoundsRandomArray:Array;
		
		public function Model() {
			_pKaLoader = new PKaLoader();
			_emailer = new Emailer();
			
			_referencesXMLListCollection = new XMLListCollection();
			_compoundsArrayCollection = new ArrayCollection();
		}
		
		public function get referencesXMLListCollection():XMLListCollection {
			return _referencesXMLListCollection;
		}
		
		public function get compoundsArrayCollection():ArrayCollection {
			return _compoundsArrayCollection;
		}
		
		public function loadData():void {
			
			// Reminder: loading is asynchronous.
			_pKaLoader.addEventListener(PKaLoader.REFERENCES_LOADED, handleReferencesLoaded, false, 0, true);
			_pKaLoader.loadReferences();
			_pKaLoader.addEventListener(PKaLoader.COMPOUNDS_LOADED, handleCompoundsLoaded, false, 0, true);
			_pKaLoader.loadCompounds();
		}
		
		// Parse the references data into a collection.
		private function handleReferencesLoaded(event:Event):void {
			
			_pKaLoader.removeEventListener(event.type, arguments.callee);
			var xml:XML = _pKaLoader.referencesXML;
			var xmlList:XMLList = xml.elements(REFERENCE_TAG);
			_referencesXMLListCollection.source = xmlList;
			
			dispatchEvent(new Event(PKaLoader.REFERENCES_LOADED) );
		}
		
		// Parse the compounds data into a collection.
		private function handleCompoundsLoaded(event:Event):void {
			
			_pKaLoader.removeEventListener(event.type, arguments.callee);
			var xml:XML = _pKaLoader.compoundsXML;
			var xmlList:XMLList = xml.elements(Compound.COMPOUND_TAG);
			
			var compound:Compound;
			var numImages:int = 0;
			for (var i:int = 0; i < xmlList.length(); i++) {
				compound = new Compound(xmlList[i]);
				_compoundsArrayCollection.addItem(compound);
				if (compound.hasImage)
					numImages++;
			}
			
			main.logThis("Compounds: " + _compoundsArrayCollection.length + ".");
			main.logThis("Compounds with images: " + numImages + ".");
			
			dispatchEvent(new Event(PKaLoader.COMPOUNDS_LOADED) );
		}
		
		// Return a random unused compound with an image. If all are used, return a new random one.
		public function getNewRandomCompoundWithAnImage():Compound {
			
			//Main.tracerView.traceText("Looking for the next random image . . .", true, 1);
			
			// Check twice the number of compounds to ensure we checked each compound at least once.
			var compound:Compound;
			for (var compoundsTried:int = 0; 
				compoundsTried < 2 * _compoundsArrayCollection.length; 
				compoundsTried++) {
				
				// If no random compounds left, make a new random order.
				if (_compoundsRandomArray == null || _compoundsRandomArray.length == 0) {
					main.logThis("Making list of compounds in random order.");
					var array:Array = _compoundsArrayCollection.toArray();
					_compoundsRandomArray = randomizeArray(array);
				}
				compound = Compound(_compoundsRandomArray.pop() );
				
				// If compound has an image, then return it.
				if (compound.hasImage) {
						return compound;
				}
			}
			
			// If there were no images, note that.
			main.logThis("Warning: No compounds have an image.");
			return null;
		}
		
		// Return the given array with its items in a random order.
		// Could be a general utility.
		private function randomizeArray(array:Array):Array {
			
			array.sort(randomSort);
			return array;
		}
		
		// Returns -1 or 1 randomly.
		// Could be a general utility.
		private function randomSort(elementA:Object, elementB:Object):Number {
			
			// [0, 1) -> 0 or 1 -> 0 or 2 -> -1 or 1 at ~50% probability.
			return (Math.round(Math.random() ) * 2) - 1;
		}
		
		// E-mail the message to the developers.
		public function emailMessage(message:Message):void {
			
			_emailer.addEventListener(StatusEvent.STATUS, handleEmailerStatus, false, 0, true);
			_emailer.sendMessage(message);
		}
		
		private function handleEmailerStatus(statusEvent:StatusEvent):void {
			
			statusEvent.currentTarget.removeEventListener(statusEvent.type, arguments.callee);
			dispatchEvent(new StatusEvent(StatusEvent.STATUS, false, false, statusEvent.code) );
		}
	}
}