package geoffhom.com.pKaValues {
	import flash.events.EventDispatcher;
	import flash.net.URLRequest;
	import flash.net.URLLoader;
	import flash.events.Event;
	import flash.system.Security;
	
	/* For loading pKa data from external files.
	The data is loaded by calling PHP scripts which load XML files.
	*/
	public class PKaLoader extends EventDispatcher {
		
		public static const COMPOUNDS_LOADED:String = "Compounds loaded.";
		public static const REFERENCES_LOADED:String = "References loaded.";

		/* Scripts which return the corresponding XML file(s) as text.
		*/
		// For testing on our development machine.
		private const REFERENCES_SCRIPT_LOCAL:String = DefaultValues.LOCAL_URL + "get-references-offline.php";
		private const COMPOUNDS_SCRIPT_LOCAL:String = DefaultValues.LOCAL_URL + "get-compounds-offline.php";		
		// For running on the server.
		private const REFERENCES_SCRIPT_REMOTE:String = DefaultValues.REMOTE_PATH + "get-references.php";
		private const COMPOUNDS_SCRIPT_REMOTE:String = DefaultValues.REMOTE_PATH + "get-compounds.php";
		
		private var _referencesXML:XML;
		private var _compoundsXML:XML;
		
		private var _referencesURLLoader:URLLoader;
		private var _compoundsURLLoader:URLLoader;
		
		public function PKaLoader() {
		}
		
		public function get referencesXML():XML {
			return _referencesXML;
		}
		
		public function get compoundsXML():XML {
			return _compoundsXML;
		}
		
		/* Load XML file containing the references.
		The XML file is loaded via a PHP script.
		The script and its location depend on the sandbox.
		*/
		public function loadReferences():void {
			
			_referencesURLLoader = new URLLoader();
			_referencesURLLoader.addEventListener(Event.COMPLETE, handleReferencesLoaded, false, 0, true);
			
			var url:String;
			if (Security.sandboxType == Security.LOCAL_TRUSTED)
				url = REFERENCES_SCRIPT_LOCAL;
			else
				url = REFERENCES_SCRIPT_REMOTE;
			var urlRequest:URLRequest = new URLRequest(url);
			
			try {
				_referencesURLLoader.load(urlRequest);
			}
			catch (error:Error) {
				main.logThis("PKaLoader: Unable to load URL:" + error);
			}
		}
		
		// Once the data are loaded, convert them to an XML object.
		private function handleReferencesLoaded(event:Event):void {
			
			try {
				event.currentTarget.removeEventListener(event.type, arguments.callee);
				_referencesURLLoader.close();
				_referencesXML = XML(_referencesURLLoader.data);
				notifyOfReferencesLoaded();
			}
			catch (error:Error) {
				main.logThis("PKaLoader: Unable to load data from URL:" + error);
			}
		}
		
		// Tell listeners that the references have been loaded from the server.
		private function notifyOfReferencesLoaded():void {
			
			dispatchEvent(new Event(PKaLoader.REFERENCES_LOADED) );
		}
		
		/* The functions for loading refs and compounds could be combined into generic functions.
		Do this if I load a third file.
		*/
		
		/* Load XML file containing the references.
		The XML file is loaded via a PHP script.
		The script and its location depend on the sandbox.
		*/
		public function loadCompounds():void {
			
			_compoundsURLLoader = new URLLoader();
			_compoundsURLLoader.addEventListener(Event.COMPLETE, handleCompoundsLoaded, false, 0, true);
			
			var url:String;
			if (Security.sandboxType == Security.LOCAL_TRUSTED)
				url = COMPOUNDS_SCRIPT_LOCAL;
			else
				url = COMPOUNDS_SCRIPT_REMOTE;
			var urlRequest:URLRequest = new URLRequest(url);
			
			try {
				_compoundsURLLoader.load(urlRequest);
			}
			catch (error:Error) {
				main.logThis("PKaLoader: Unable to load URL:" + error);
			}
		}
		
		// Once the data are loaded, convert them to an XML object.
		private function handleCompoundsLoaded(event:Event):void {
			
			try {
				event.currentTarget.removeEventListener(event.type, arguments.callee);
				_compoundsURLLoader.close();
				_compoundsXML = XML(_compoundsURLLoader.data);
				notifyOfCompoundsLoaded();
			}
			catch (error:Error) {
				main.logThis("PKaLoader: Unable to load data from URL:" + error);
			}
		}
		
		// Tell listeners that the references have been loaded from the server.
		private function notifyOfCompoundsLoaded():void {
			
			dispatchEvent(new Event(PKaLoader.COMPOUNDS_LOADED) );
		}
	}
}